#!/usr/bin/perl
use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request::Common qw/ POST /;
use Getopt::Long qw(:config no_ignore_case);
use Pod::Usage;
use Digest::MD5 qw/ md5_hex /;

### [ some "constants" | #### {{{
my %TYPE=(
    hdd =>  6,
    vcr =>  1,
    rem =>  0,
    wake => 5,
);
my %TYPE_NUM;
@TYPE_NUM{values %TYPE}= keys %TYPE;

my %REPEAT=(
    none     => 0,
    daily    => 1,
    weekly   => 2,
    weekdays => 3,
    weekend  => 4,
);
my %REPEAT_NUM;
@REPEAT_NUM{values %REPEAT}= keys %REPEAT;

###}}}

### [ get parameters ] ### {{{
my (
    $command,
    $error,
    $host,
    $channel,
    $day,
    $start,
    $end,
    $pass,
    $type,   
    $repeat,
);
my %COMMANDS= (
    set      => { host => 1, channel => 1, date => 1 },
    delete   => { host => 1, channel => 1, date => 1, },
    channels => { host => 1, },
);
help() unless GetOptions(
    'H|host=s'     => \$host,
    'c|channel=s'  => \$channel,
    'd|day=s'      => \$day,
    's|start=s'    => \$start,
    'e|end=s'      => \$end,
    'p|password=s' => \$pass,
    't|type=s'     => \$type,   
    'r|repeat=s'   => \$repeat,
    'h|help'     => \&help,
    'm|man'      => \&man,
);

sub help { pod2usage(-verbose=>1); exit; }
sub man  { pod2usage(-verbose=>2); exit; }
### }}}

### [ Parameter check ] ### {{{

## Command check
$command= lc shift;
do_die(1, "Missing command.\n")         unless $command;
my $required= $COMMANDS{$command};
do_die(1, "Unknown command $command\n") unless defined $required;

## Required parameter check
error("Missing host.")       if $required->{'host'}    and not $host;
error("Missing channel.")    if $required->{'channel'} and not $channel;
error("Missing day.")        if $required->{'date'}    and not $day;
error("Missing starttime.")  if $required->{'date'}    and not $start;
error("Missing endtime.")    if $required->{'date'}    and not $end;

## Parameter validation
error("Wrong day $day.")         if $day
                                 and not $day=~ /^\d?\d\.\d?\d\.$/;
error("Wrong starttime $start.") if $start
                                 and not $start=~ /^\d?\d:\d?\d$/;
error("Wrong endtime $end.")     if $end
                                 and not $end=~ /^\d?\d:\d?\d$/;
if (defined $type) {
    if ($TYPE_NUM{$type}) {
        $type= $TYPE_NUM{$type};
    }
}
else {
    $type='hdd';
}
error("Unknown type $type.") unless defined $TYPE{$type};
if (defined $repeat) {
    if ($REPEAT_NUM{$repeat}) {
        $repeat= $REPEAT_NUM{$repeat};
    }
}
else {
    $repeat='none';
}
error("Unknown REPEAT $repeat.") unless defined $REPEAT{$repeat};
do_die(1, $error) if $error;

$pass= $pass ? md5_hex($pass).'_' : '';

# Fix date/time
if ($required->{'date'}) {
    s/\b(?=\d\b)/0/g for $day, $start, $end;
}
###}}}

SWITCH: for ($command) {
/^delete$/ && do {
    ### [ retrieve timer list ] ### {{{
    my $ua= new LWP::UserAgent;
    
    my $url= "http://$host/index_s.html?${pass}timeroverview=_";
    
    my $req = new HTTP::Request 'GET' => $url;
    $req->header('Accept' => 'text/html');
    
    # send request
    my $res = $ua->request($req);
    
    # check the outcome
    do_die(10, "Error: " . $res->status_line . "\n") unless $res->is_success;
    my $q_channel = quotemeta $channel ;
    my $q_day     = quotemeta $day     ;
    my $q_start   = quotemeta $start   ;
    my $q_end     = quotemeta $end     ;  
    $res->content=~ m!
        <tr>
        <td\b[^>]*>[^:]+:\s*(?i:$q_channel)</td>
        <td\b[^>]*>$q_day\d+</td>
        <td\b[^>]*>$q_start</td>
        <td\b[^>]*>$q_end</td> 
        .*
        name='(${pass}deletetimer_\d+)'
    !x or do_die(7, "Timer not found.\n");        
    ###}}}
    
    ### [ Send delete command ] ### {{{
    $url= "http://$host/index_s.html?$1=_";
    
    $req = new HTTP::Request 'GET' => $url;
    $req->header('Accept' => 'text/html');
    
    # send request
    $res = $ua->request($req);
    
    # check the outcome
    do_die(8, "Error: " . $res->status_line . "\n") unless $res->is_success;
    ###}}}
        
    ### [ Confirm delete ] ### {{{
    $url= "http://$host/index_s.html?${pass}confirmdelete=_";
    
    $req = new HTTP::Request 'GET' => $url;
    $req->header('Accept' => 'text/html');
    
    # send request
    $res = $ua->request($req);
    
    # check the outcome
    do_die(9, "Error: " . $res->status_line . "\n") unless $res->is_success;
    print "Timer $day $start - $end $channel deleted\n";
    ###}}}
    last SWITCH;
    };
/^set$/ && do { # set timer
    ### [ retrieve channel list ] ### {{{
    my $channels= get_channels();
    my $channel_num;
    my $uc_channel= uc $channel;
    foreach (keys %$channels) {
        next unless $uc_channel eq uc $_;
        $channel_num= $channels->{$_};
        last;
    }
    do_die(3,"Channel $channel not found\n") unless defined $channel_num;

    ###}}}
    
    ### [ set the timer ] ### {{{

    my $ua= new LWP::UserAgent;
    my $url= "http://$host/index_s.html?${pass}new${type}timer=_";
    my $req = POST $url, [ 
    service_1  => $channel_num,
     date      => $day,
     start     => $start,     
     stop      => $end,     
     repeat    => $REPEAT{$repeat}, 
     type      => $TYPE{$type},
     "${pass}set_newtimer" => '_',
    ];
    
    my $res= $ua->request($req);
    if ($res->is_success) {
        if ($res->content=~ m#<h4><.*?>(.*?)</.*?></h4>#) {
            do_die(5, $1."\n");
        }
        else {
            print "OK\n";
        }
    }
    else {
        do_die(6,"Error: " . $res->status_line . "\n");
    }
    ###}}}
    last SWITCH;
    };
/^channels$/ && do {
    my $channels= get_channels();
    print "$_\n" foreach (sort {$channels->{$a} <=> $channels->{$b} } keys %$channels);
    last SWITCH;
    };
}

### [ Helpers ] ###{{{

sub error {
    $error.= (shift) . "\n";
}

sub do_die {
    my $exit_code= shift;
    print STDERR shift;
    exit $exit_code;
}

sub get_channels {
    my $ua= new LWP::UserAgent;
    
    my $url= "http://$host/index_s.html?${pass}new${type}timer=_";
    
    my $req = new HTTP::Request 'GET' => $url;
    $req->header('Accept' => 'text/html');
    
    # send request
    my $res = $ua->request($req);
    
    # check the outcome
    my %channels;
    if ($res->is_success) {
        $res->content=~ m:<select\b[^>]+\bname='service_1'[^>]*>(.*?)</select>:s or do_die(2, "No channel list found.\n");
        # Extract the channels
        $_= $1;
        while (m#<option\b[^>]*\bvalue='(\d+)'[^>]*>\d+\s*(.*?)</option>#g) {
            $channels{$2}= $1;
        }
    }
    else {
        do_die(4, "Error: " . $res->status_line . "\n");
    }
    return \%channels;
}

###}}}

### [ man page ] ###{{{

=head1 NAME

technistar.pl - a tool to program recordings on a TechniStar S2 receiver

=head1 SYNOPSIS

technistar.pl [command] -h host -d day -s starttime -e endtime -c channelname ...

=head1 DES‎CRIP‎TION

technistar.pl will send the provided information to a Technistar S2
receiver to program a recording.

=head1 COMMANDS

=over 4

=item set

Set a timer. Requires a start and end time as well as a channel to be
given.

=item delete

Deletes a timer. Requires the same parameters as the L<set> command.

=item channels

Will list all channels.

=back

=head1 OPTIONS

=over 4


=item -c B<channelname>

=item --channel B<channelname>

The channel name.


=item -d B<day>

=item --day B<day>

The start day in the form "d.m.".


=item -e B<endtime>

=item --end B<endtime>

The end time in the form H:MM.


=item -H B<host(:port)>

=item --host B<host(:port)>

The hostname or IP address of the receiver, optionally followed by
a colon and the portnumber (default 80).


=item -p B<password>

=item --pass B<password>

The password - if any


=item -s B<starttime>

=item --start B<starttime>

The end time in the form H:MM.


=item -r B<repeat>

=item --repeat B<repeat>

how to repeat the recording:

=over 2

=item none     or 0 (default)

=item daily    or 1

=item weekly   or 2

=item weekdays or 3

=item weekend  or 4

=back


=item -t B<type>

=item --type B<type>

The type of the timer:

=over 2

=item hdd or 6 (default)

A hard disc recording timer.

=item rem or 0

A channel switching timer.

=item vcr or 1

A videorecorder timer.

=item wake or 5

A wake timer.

=back

=back

=head1 BUGS

none yet

=head1 TODO

Make it less "Quick and Dirty" - More OO.

=head1 AUTHOR

Stephan Hradek technistar2012.v1.hradek@xoxy.net

=cut

###}}}